<?php
/**
 * @name Server
 * @description Classe de base gérant les serveurs du plugin "Server"
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 07/11/2012
 */
include_once SNMP_CLASS;
include_once LIB_SNMP;
include_once DEVICE_CLASS;
class FlowServerException extends DeviceException{ }
class FlowServer extends Device
{
	const OUTPUT_JSON = 1;
	const OUTPUT_OBJECT = 2;
	const OUTPUT_ARRAY = 3;
	const DEBUG_NONE = 0;
	const DEBUG_ENABLE = 1;
	protected $url;
	protected $debug;
	/*@NAME: __construct
	 *@IN: $dev un objet Device
	*@OUT: /
	*@TREAT: permet de construire un objet SyslogServer à partir d'un objet "Device"
	*@EXCEPTION: 	/
	*/
	public function __construct($dev=NULL)
	{
		$this->debug= FlowServer::DEBUG_NONE;
		if(get_class($dev) == 'Device')
		{
			parent::constructDevice($dev);
		}
		else
		{
			parent::__construct();
		}
	}
	public function setUrl($u)
	{
		$this->url = $u;
	}
	public function getUrl()
	{
		return $this->url;
	}
	public function setDebugMode($mode)
	{
		if($mode == FlowServer::DEBUG_ENABLE)
		{
			$this->debug=FlowServer::DEBUG_ENABLE;
		}
		else
		{
			$this->debug= FlowServer::DEBUG_NONE;
		}
	}
	private function formatOutput($data,$output)
	{
		if($output == FlowServer::OUTPUT_JSON)
		{
			return $data;
		}
		elseif($output == FlowServer::OUTPUT_ARRAY)
		{
			return json_decode($data,TRUE);
		}
		else
		{
			return json_decode($data);
		}
	}
	private function getHTTPRequest($uri)
	{
		if($this->debug == FlowServer::DEBUG_ENABLE)
		{
			echo 'DEBUG MODE: HTTP REQUEST '.$this->url."/$uri\n<br />";
		}
		$data = file_get_contents($this->url."/$uri");
		if($data === FALSE)
		{
			$err = error_get_last();
			throw new FlowServerException("FLOWS server HTTP ERROR: ".$err['message']);
		}
		return $data;
	}
	private function postHTTPRequest($uri,$params)
	{
		if($this->debug == FlowServer::DEBUG_ENABLE)
		{
			echo 'DEBUG MODE: POST REQUEST '.$this->url."/$uri\n<br />";
		}
		$postclean = http_build_query($params);
		$curl = curl_init();
		curl_setopt($curl,CURLOPT_URL, $this->url."/$uri");
		curl_setopt($curl,CURLOPT_POST, true);
		curl_setopt($curl,CURLOPT_POSTFIELDS, $postclean);
		curl_setopt($curl,CURLOPT_RETURNTRANSFER, true); 
		$response = curl_exec($curl);
		if($response === FALSE)
		{
			$err = curl_error($curl);
			throw new FlowServerException("FLOWS server POST HTTP ERROR: ".$err);
		}
		curl_close($curl);
		return $response;
	}
	public function getCollectorsStatistic($channelid=null,$output=FlowServer::OUTPUT_OBJECT)
	{
		$params=null;
		if(!empty($channelid))
		{
			
		}
		//echo $this->url."/collector.php?action=lastdata&name=$name";
		$stat = $this->getHTTPRequest("stats.php?action=collectors$params");
		return $this->formatOutput($stat,$output);
	}
	public function getChannel($channelid=null,$output=FlowServer::OUTPUT_OBJECT)
	{
		$params="";
		if(!empty($channelid))
		{
			$params="&id=$channelid";
		}
		$stat = $this->getHTTPRequest("channel.php?action=getChannel$params");
		return $this->formatOutput($stat,$output);
	}
	public function getChannelStatistic($channelid=null,$type='bytes',$output=FlowServer::OUTPUT_OBJECT)
	{
		$params="type=$type";
		if(!empty($channelid))
		{
			$params="&id=$channelid";
		}
		$stat = $this->getHTTPRequest("stats.php?action=traffic&$params");
		return $this->formatOutput($stat,$output);
	}
	public function getCollector($channel='all',$output=FlowServer::OUTPUT_OBJECT)
	{
		$params="";
		if($channel !='all')
		{
			$params="&channel=$channel";
		}
		$stat = $this->getHTTPRequest("collector.php?action=get".$params);
		return $this->formatOutput($stat,$output);
	}
	public function getCollectorPlugin($collector='all',$output=FlowServer::OUTPUT_OBJECT)
	{
		$params="";
		if($collector !='all')
		{
			$params="&collector=$collector";
		}
		$stat = $this->getHTTPRequest("collector.php?action=getPlugin".$params);
		return $this->formatOutput($stat,$output);
	}
	public function getDataCollector($collectorid,$dataid=null,$output=FlowServer::OUTPUT_OBJECT)
	{
		$data="";
		if(!empty($dataid))
		{
			$data="&dataid=$dataid";
		}
		//echo $this->url."/collector.php?action=data&id=$collectorid".$data;
		$stat = $this->getHTTPRequest("collector.php?action=data&id=$collectorid".$data);
		return $this->formatOutput($stat,$output);
	}
	public function executeCollector($ip,$name = 'TOP_10_IP_BYTES',$data='global',$type='all',$during='15min',$channel="",$output=FlowServer::OUTPUT_OBJECT)
	{
		$plugin = $name;
		if($data=='lan')
		{
			$plugin .= '_LAN';
		}
		elseif($data=='public')
		{
			$plugin .= '_PUBLIC';
		}
		if($type=='upload')
		{
			$plugin .= '_UPLOAD';
		}
		elseif($type=='download')
		{
			$plugin .= '_DOWNLOAD';
		}
		switch($during)
		{
			case '15min':$plugin .='_15MIN'; break;
			case '1h':$plugin .='_1H'; break;
			case '6h':$plugin .='_6H'; break;
			case '1d':$plugin .='_1D'; break;
			case '7d':$plugin .='_7D'; break;
			case '30d':$plugin .='_30D'; break;
			default: return null ; break;
		}
        $params = "plugin=$plugin";
        $params .= "&IP=$ip";
        if(!empty($channel))
		{
			$params.="&channel=$channel";
		}
		$stat = $this->getHTTPRequest("collector.php?action=execute&".$params);
		return $this->formatOutput($stat,$output);
	}
	public function getLastTopIP($data='global',$type='all',$during='15min',$channel='all',$output=FlowServer::OUTPUT_OBJECT)
	{
		$name = 'TOP_10_IP_BYTES';
		if($data=='lan')
		{
			$name = 'TOP_10_IP_BYTES_LAN';
		}
		elseif($data=='public')
		{
			$name = 'TOP_10_IP_BYTES_PUBLIC';
		}
		if($type=='upload')
		{
			$name .= '_UPLOAD';
		}
		elseif($type=='download')
		{
			$name .= '_DOWNLOAD';
		}
		switch($during)
		{
			case '15min':$name .='_15MIN'; break;
			case '1h':$name .='_1H'; break;
			case '6h':$name .='_6H'; break;
			case '1d':$name .='_1D'; break;
			case '7d':$name .='_7D'; break;
			case '30d':$name .='_30D'; break;
			default: return null ; break;
		}
		//echo $this->url."/collector.php?action=lastdata&name=$name";
		$stat = $this->getHTTPRequest("collector.php?action=lastdata&name=$name");
		return $this->formatOutput($stat,$output);
	}
	public function getTopPort($during='15min',$channel='all',$output=FlowServer::OUTPUT_OBJECT)
	{
		$name = 'TOP_10_PORT';
		switch($during)
		{
			case '15min':$name .='_15MIN'; break;
			case '1h':$name .='_1H'; break;
			case '6h':$name .='_6H'; break;
			case '1d':$name .='_1D'; break;
			case '7d':$name .='_7D'; break;
			case '30d':$name .='_30D'; break;
			default: return null ; break;
		}
		$stat = $this->getHTTPRequest("/collector.php?action=lastdata&name=$name");
		if($stat === FALSE)
		{
			$err = error_get_last();
			throw new FlowServerException("Host not found: ".$err['message']);
		}
		return $this->formatOutput($stat,$output);
	}
	public function addCaptureChannel($name,$ip,$protocol,$port)
	{
		$params = array('ip_dist'=>$ip,'name'=>$name,'protocol'=>$protocol,'port'=>$port);
		$this->postHTTPRequest('channel.php?action=addChannel',$params);
		
	}
}
